LIBRARY({
    name: "StalkerAPI",
    version: 2.1,
    shared: true,
    api: "CoreEngine",
});

var _LogError = Logger.LogError;
Logger.LogError = function(a,b){
	alert("[ERROR]{"+b+"}"+a);
	_LogError(a,b);
}

var StalkerDev = {
	addGun:function(a){
		this.mod_gun.push(a.id);
	},
	addAmmo:function(a){
		this.mod_ammo.push(a.id);
	},
	addArmor:function(a){
		this.mod_armor.push(a.id);
	},
	createGun:function(){
		Callback.invokeCallback("StalkerAddGun");
	},
	createAmmo:function(){
		Callback.invokeCallback("StalkerAddAmmo");
	},
	createArmor:function(){
		Callback.invokeCallback("StalkerAddArmor");
	},
	mod_gun:[],
	mod_ammo:[],
	mod_armor:[],
	gun:[],
	ammo:[],
	armor:[],
}

var Guns = {
	ShootingMode:{
		AUTO:-1,
		SINGLE:1,
		DOUBLE:2,
		TRIPLE:3
	},
	
	ShotType:{
		NORMAL:1,
		SINGLE:1,
		SHOTGUN:2,
		MULTIPLE:2
	},
	
	MAX_DAMAGE:-1,
	
	getGun:function(sid){
		for(var i = 0; i < StalkerDev.gun.length; i++){
			if(StalkerDev.gun[i].id == sid)
				return StalkerDev.gun[i];
		}
		
		return false;
	},
	
	addGun:function(gun){
		Callback.addCallback("StalkerAddGun", function(){
			
		var ammo_error = false;
		var ammo_error_str = '';
		var ammo_bullet = false;
		
		if(typeof gun.id != "string")
			return Logger.LogError("{addGun} gun.id должен быть строкой", "StalkerAPI");
		if(Guns.getGun(gun.id) || StalkerDev.mod_gun.indexOf(gun.id) != -1)
			return Logger.LogError("{addGun} оружие с id "+gun.id+" уже существует", "StalkerAPI");
		if(typeof gun.name != "string")
			return Logger.LogError("{addGun} gun.name должен быть строкой", "StalkerAPI");
		
		if(typeof gun.ammo == 'string')
			gun.ammo = [gun.ammo];
		else if(!gun.ammo instanceof Array)
			return Logger.LogError("{addGun} gun.ammo должен быть строкой или массивом строк", "StalkerAPI");
		
		for(var i = 0; i < gun.ammo.length; i++){
			if(Ammo.getAmmo(gun.ammo[i]) == false && StalkerDev.mod_ammo.indexOf(gun.ammo[i]) == -1){
				ammo_error = true;
				ammo_error_str+="gun.ammo[i],";
				continue;
			}
			if(Ammo.getBulletAmmo(gun.ammo[i])){
				ammo_bullet=true;
			}
		}
		
		if(ammo_error)
			return Logger.LogError("{addGun} Не существует боеприпасов со следующими ID:"+ammo_error_str, "StalkerAPI");
		
		if(typeof gun.accuracy != "number")
			return Logger.LogError("{addGun} gun.accuracy должен быть числом", "StalkerAPI");
		
		if(typeof gun.recoil != "number")
			return Logger.LogError("{addGun} gun.recoil должен быть числом", "StalkerAPI");
		if(typeof gun.rate != "number")
			return Logger.LogError("{addGun} gun.rate должен быть числом", "StalkerAPI");
		if(typeof gun.smoke != "number")
			return Logger.LogError("{addGun} gun.smoke должен быть числом", "StalkerAPI");
		if(gun.shotType != Guns.ShotType.NORMAL && gun.shotType != Guns.ShotType.SHOTGUN )
			return Logger.LogError("{addGun} gun.shotType должен быть ShotType.NORMAL или ShotType.SHOTGUN", "StalkerAPI");
		
		if(typeof gun.shootingModes == "number")
			gun.shootingModes = [gun.shootingModes];
		else if(gun.shootingModes instanceof Array){
			var tmp, exit = false;
			for(var i = 0; i < gun.shootingModes.length; i++){
				if(exit) break;
				
				for(var j = 0; j < gun.shootingModes.length; j++){
					if( (j+1) == gun.shootingModes.length)
						exit = true;
					
					if(gun.shootingModes[j] > gun.shootingModes[j+1]){
						tmp = gun.shootingModes[j];
						gun.shootingModes[j] = gun.shootingModes[j+1];
						gun.shootingModes[j+1] = tmp;
					}
				}
			}
		}else
			return Logger.LogError("{addGun} gun.shootingModes должен быть массивом", "StalkerAPI");
			
		
		if(typeof gun.texture == 'string')
			gun.texture = {name:gun.texture};
		else if(typeof gun.texture != 'object')
			return Logger.LogError("{addGun} gun.texture должен быть объектом", "StalkerAPI");
		
		if(typeof gun.texture.name !='string')
			return Logger.LogError("{addGun} gun.texture.name должен быть строкой", "StalkerAPI");
		
		if(typeof gun.bullet != 'object')
			return Logger.LogError("{addGun} gun.bullet должен быть объектом", "StalkerAPI");
		if(typeof gun.bullet.damage != "number")
			return Logger.LogError("{addGun} gun.bullet.damage должен быть числом", "StalkerAPI");
		if(typeof gun.bullet.speed != "number")
			return Logger.LogError("{addGun} gun.bullet.speed должен быть числом", "StalkerAPI");
		if(gun.bullet.texture && typeof gun.bullet.texture != "string")
			return Logger.LogError("{addGun} gun.bullet.texture должен быть строкой", "StalkerAPI");
		
		if(ammo_bullet)
			if(!gun.bullet.count || typeof gun.bullet.count != "number")
				return Logger.LogError("{addGun} gun.bullet.count должен быть числом", "StalkerAPI");
		
		if(typeof gun.sounds != 'string')
			return Logger.LogError("{addGun} gun.sounds должен быть строкой", "StalkerAPI");
		
		if(typeof gun.fov == 'number')
			gun.fov = {level:gun.fov};
		else if(typeof gun.fov != 'object')
			return Logger.LogError("{addGun} gun.fov должен быть объектом", "StalkerAPI");
		
		if(typeof gun.fov.level !='number')
			return Logger.LogError("{addGun} gun.fov.level должен быть числом", "StalkerAPI");
		if(gun.fov.link && typeof gun.fov.link != "string")
			return Logger.LogError("{addGun} gun.fov.link должен быть строкой", "StalkerAPI");
		
		if(gun.shotType == Guns.ShotType.SHOTGUN){
			if(typeof gun.shotgun != 'object')
				return Logger.LogError("{addGun} gun.shotgun должен быть объектом", "StalkerAPI");
			if(typeof gun.shotgun.count != 'number')
				return Logger.LogError("{addGun} gun.shotgun.count должен быть числом", "StalkerAPI");
			if(typeof gun.shotgun.degreesSpread != 'number')
				return Logger.LogError("{addGun} gun.shotgun.degreesSpread должен быть числом", "StalkerAPI");
			
		}
		
		if(granade){
			if(typeof gun.granade != 'object')
				return Logger.LogError("{addGun} gun.granade должен быть объектом", "StalkerAPI");
			if(typeof gun.granade.sounds != 'string')
				return Logger.LogError("{addGun} gun.granade.sounds должен быть строкой", "StalkerAPI");
			if(!gun.granade.count){
				gun.granade.count = 1
			}else{
				if(typeof gun.granade.count != 'number')
					return Logger.LogError("{addGun} gun.shotgun.count должен быть числом", "StalkerAPI");
			}
		}
		
		StalkerDev.gun.push(gun);
		});
	},
	
	
	addGuns:function(arrayGuns){
		for(var i = 0; i < arrayGuns.length; i++)
			Guns.addGun(arrayGuns[i]);
	},
};

var Ammo = {
	Type:{
		MAGAZIN:1,
		BULLET:2
	},
	
	getAmmo:function(sid){
		for(var i = 0; i < StalkerDev.ammo.length; i++){
			if(StalkerDev.ammo[i].id == sid)
				return StalkerDev.ammo[i];
		}
		
		return false;
	},
	
	getBulletAmmo:function(ammo){
		if(Ammo.getAmmo(ammo).type == Ammo.Type.BULLET)
			return true;
		
		return false;
	},
	
	addAmmo:function(ammo){
		Callback.addCallback("StalkerAddAmmo", function(){
			if(typeof ammo.id != "string")
				return Logger.LogError("{addAmmo} ammo.id должен быть строкой", "StalkerAPI");
			if(Ammo.getAmmo(ammo.id) || StalkerDev.mod_ammo.indexOf(ammo.id) != -1)
				return Logger.LogError("{addAmmo} боеприпасы с id "+ammo.id+" уже существует", "StalkerAPI");
			if(typeof ammo.name != "string")
				return Logger.LogError("{addAmmo} ammo.name должен быть строкой", "StalkerAPI");
			if(ammo.gun_name && typeof ammo.gun_name != 'string')
				return Logger.LogError("{addAmmo} ammo.gun_name должен быть строкой", "StalkerAPI");
			
			if(typeof ammo.texture == 'string')
				ammo.texture = {name:ammo.texture};
			else if(typeof ammo.texture != 'object')
				return Logger.LogError("{addAmmo} ammo.texture должен быть объектом", "StalkerAPI");
			if(ammo.texture.prefix && typeof ammo.texture.prefix != 'string')
				return Logger.LogError("{addAmmo} ammo.texture.prefix должен быть строкой", "StalkerAPI");
			
			if(ammo.type != Ammo.Type.MAGAZIN && ammo.type != Ammo.Type.BULLET)
				return Logger.LogError("{addAmmo} ammo.type должен быть Ammo.Type.MAGAZIN или Ammo.Type.BULLET", "StalkerAPI");
			
			if(ammo.type == Ammo.Type.MAGAZIN && typeof ammo.count != 'number')
				return Logger.LogError("{addAmmo} ammo.count должен быть числом", "StalkerAPI");
			
			StalkerDev.ammo.push(ammo);
		});
	},

	addAmmos:function(arrayAmmos){
		for(var i = 0; i < arrayAmmos.length; i++)
			Ammo.addAmmo(arrayAmmos[i]);
	},
}

var Armor = {
	Type:{
		ARMOR:1,
		HELMET:2,
		FULLARMOR:3
	},
	
	getArmor:function(sid){
		for(var i = 0; i < StalkerDev.armor.length; i++){
			if(StalkerDev.armor[i].id == sid || StalkerDev.armor[i].id+"_helmet" == sid || StalkerDev.armor[i].id+"_chestplate" == sid)
				return StalkerDev.armor[i];
		}
		
		return false;
	},
	
	addArmor:function(armor){
		Callback.addCallback("StalkerAddArmor", function(){
			if(typeof armor.id != "string")
				return Logger.LogError("{addArmor} armor.id должен быть строкой", "StalkerAPI");
			if(Armor.getArmor(armor.id) || StalkerDev.mod_armor.indexOf(armor.id) != -1)
				return Logger.LogError("{addArmor} броня с id "+armor.id+" уже существует", "StalkerAPI");
			if(typeof armor.name != "string")
				return Logger.LogError("{addArmor} armor.name должен быть строкой", "StalkerAPI");
			
			if(armor.type != Armor.Type.HELMET && armor.type != Armor.Type.ARMOR && armor.type != Armor.Type.FULLARMOR)
				return Logger.LogError("{addArmor} armor.type должен быть Armor.Type.ARMOR, Armor.Type.HELMET или Armor.Type.FULLARMOR", "StalkerAPI");
			
			if(typeof armor.armor != 'number')
				return Logger.LogError("{addArmor} armor.armor должен быть числом", "StalkerAPI");
			if(typeof armor.durability != 'number')
				return Logger.LogError("{addArmor} armor.durability должен быть числом", "StalkerAPI");
			
			if(typeof armor.texture == 'string')
				armor.texture = {name:armor.texture};
			else if(typeof armor.texture != 'object')
				return Logger.LogError("{addArmor} armor.texture должен быть объектом", "StalkerAPI");
			
			if(armor.texture.model && typeof armor.texture.model != 'string')
				return Logger.LogError("{addArmor} armor.texture.model должен быть строкой", "StalkerAPI");
			
			StalkerDev.armor.push(ammo);
		});
	},
	
	addArmors:function(arrayArmors){
		for(var i = 0; i < arrayArmors.length; i++)
			Armor.addArmor(arrayArmors[i]);
	}
};

var Stalker = {
	init:function(dir){
		var addon_dir = dir;
		var mod_dir = __dir__;
		
		
		var config = JSON.parse(FileTools.ReadText(addon_dir+"stalkerLoadFiles.json"));
		if(config.copy == true)
			return;
		
		if(!config.dev)config.dev=false;
		if(!config.ignore)config.ignore=[];
		
		var resources = {};
		var _resources = JSON.parse(FileTools.ReadText(addon_dir+"build.config")).resources;
		for(var i = 0; i < _resources.length; i++){
			resources[_resources[i].resourceType] = _resources[i].path;
		}
		
		if(new java.io.File(addon_dir+"/sounds/").exists())
			copyFolder(addon_dir+"sounds/", mod_dir+"sounds/", config.ignore);
		if(new java.io.File(addon_dir+"/"+resources.gui+"crosshair/").exists())
			copyFolder(addon_dir+resources.gui+"crosshair/", mod_dir+"gui/crosshair/", config.ignore);
		
		if(config.dev==false){
			config.copy=true;
			FileTools.WriteText(addon_dir+"stalkerLoadFiles.json", JSON.stringify(config));
		}
	},
};

function copyFolder(outPath, inPath, ignore){
	if(!ignore) ignore = [];
	var files = new java.io.File(outPath).list();
	for(var i = 0; i < files.length; i++){
		if(indexOf(ignore, outPath+files[i]) != -1)
			continue;
		
		var file = new java.io.File(outPath+files[i]);
		var file_mod = new java.io.File(inPath+files[i]);
		if(file.isDirectory()){
			file_mod.mkdir();
			copyFolder(outPath+files[i]+"/", inPath+files[i]+"/", ignore);
		}else{
			if(file_mod.exists())continue;
			
			file_mod.createNewFile();
			copyFile(file, file_mod);
			if(config.dev==false)file.delete();
		}
	}
}

function copyFile(sourceFile, destFile){
	if (!sourceFile.exists()) {
		return Logger.LogError("Ошибка копирования файлов", "StalkerAPI");
	}
	var source = new java.io.FileInputStream(sourceFile).getChannel();
	var destination = new java.io.FileOutputStream(destFile).getChannel();
	
	if (destination != null && source != null) {
		destination.transferFrom(source, 0, source.size());
	}
	if (source != null) {
		source.close();
	} 
	if (destination != null) {
		destination.close();
	}
	Logger.Log("Файл "+sourceFile.getAbsolutePath()+" скопирован в "+destFile.getAbsolutePath(), "StalkerAPI");
}

function indexOf(array, value, index){
	if(!index)index=0;
	for(var i = index; i < array.length; i++)
		if(new RegExp(array[i].replace(new RegExp("([\\\\|\\.|\\/])","g"), "\\$1")+"$").test(value))return i;
	
	return -1;
}

EXPORT("Stalker", Stalker);
EXPORT("Guns", Guns);
EXPORT("Ammo", Ammo);
EXPORT("Armor", Armor);

EXPORT("StalkerDev", StalkerDev);